/**
   @file    Dot_Matrix_Sample.ino
   @brief   Simple program of displaying a text string - V1.0
   @author  Thomas Wiemken, eQ-3 Entwicklung GmbH
   <OPTIONAL: LONG DESCRIPTION>
**/
/*-----------------------------------------------------------------------------
   Copyright 2012 eQ-3 Entwicklung GmbH
   All Rights Reserved.

   The information contained in this file is confidential property of "eQ-3
   Entwicklung GmbH". The use copying, transfer or disclosure is prohibited
   except by written agreement with "eQ-3 Entwicklung GmbH".
  ---------------------------------------------------------------------------*/

/** @addtogroup Dot_Matrix_Sample
    @{
**/
/*---------------------------------------------------------------------------*/


/**
    Includes -----------------------------------------------------------------
**/
#include "PAD4_DM.h"


/**
    Defines -----------------------------------------------------------------
**/
// How many display modules are attached to the Arduino?
// Each module contains two digits
#define NUMBER_OF_MODULES   2

// Declare the pins for controlling the shift registers
#define SERIAL_DATA_PIN     2
#define SERIAL_CLOCK_PIN    3
#define SERIAL_CLEAR_PIN    4
#define REGISTER_CLOCK_PIN  5
#define OUTPUT_ENABLE_PIN   6

// Declare the Display_Module object
PAD4_DM Display_Module;

// Pointer to the text array
char* s8_text;


/**
    Functions ----------------------------------------------------------------
**/

void setup() {
  // put your setup code here, to run once:
  
  // Initialize  the Display_Module object
  Display_Module.begin(NUMBER_OF_MODULES, SERIAL_DATA_PIN, SERIAL_CLOCK_PIN, SERIAL_CLEAR_PIN, REGISTER_CLOCK_PIN, OUTPUT_ENABLE_PIN );
  
  /*
   * Create an array for the string text, depending on the number of used modules.
   * Always one character more than needed for the NULL-Termination '\0'
   */
  s8_text = (char*)calloc(((Display_Module.get_number_of_digits()) + 1), sizeof(char));

  // Create a simple string with a length depending on the number of used modules
  for ( unsigned char u8_index = 0; u8_index < Display_Module.get_number_of_digits(); u8_index++ )
  {
    s8_text[u8_index] = ('0' + u8_index); // s8_text = "0123..."
  }

  // Create the bit patterns of the string text and copy them into the display buffer
  Display_Module.set_display_buffer( s8_text );
  
  // Sets the digital pin 13 as output for the LED
  pinMode(13, OUTPUT);
}

void loop() {
  // put your main code here, to run repeatedly:

  static unsigned char u8_led_counter = 0;
  static unsigned char u8_led_status = 0;
  static unsigned char u8_display_tick = 0;

  if (u8_led_counter++ >= 150)  // Check and increment the LED counter variable
  {
    u8_led_counter = 0;  // Reset the counter variable for the LED blinking

    if ( 1 == u8_led_status)
    {
      digitalWrite(13, LOW);  // sets the digital pin 13 off
      u8_led_status = 0;
    }
    else
    {
      digitalWrite(13, HIGH); // sets the digital pin 13 on
      u8_led_status = 1;

      // Set the display tick variable
      u8_display_tick = 1;
    }
  }

  if (1 == u8_display_tick ) // Check the display tick variable
  {
    u8_display_tick = 0;  // Reset the display tick variable
    
    // Increment the character of each digit in the string
    for ( unsigned char u8_index = 0; u8_index < Display_Module.get_number_of_digits(); u8_index++ )
    {
      // Has the digit reached the last character
      if ( s8_text[u8_index] != 'Z' )
      {
        s8_text[u8_index]++; // No, increase the character of the digit
      }
      else
      {
        s8_text[u8_index] = '0'; // Yes, set back to character '0'
      }
    }

    // overwrite the display buffer with the new string
    Display_Module.set_display_buffer( s8_text );
  }

  // Polling the display hardware
  Display_Module.update_matrix(  );

  delayMicroseconds(500);   // waits for 500 mikroseconds
}
